'''
Created on May 30, 2021

@author: shuangyinli
'''
import lightgbm as lgb
from numpy import *
import numpy as np
import operator
import random
import copy
import sys
import re
import time
import os
from multiprocessing import Process, Manager
from copy import deepcopy
from collections import Counter

from sklearn.metrics import precision_score, recall_score, f1_score,accuracy_score
from sklearn.model_selection import cross_validate
from sklearn.model_selection import GridSearchCV
from sklearn.model_selection import train_test_split, StratifiedKFold
from sklearn import metrics
import pandas as pd
from sklearn.model_selection import KFold
from sklearn.model_selection import train_test_split
from sklearn import datasets
    
biswtmcodes = []
categories = []
biswtmfeatures = []



def kfold(X,Y, n_splits=5):
    kf = KFold(n_splits,shuffle=True,random_state=None)

    f1Num = []
    accNum = []
    scoreNum =[]
    for k, (train, test) in enumerate(kf.split(X,Y)):
        X_train = X.iloc[train]
        X_test = X.iloc[test]
        Y_train = Y.iloc[train]
        Y_test = Y.iloc[test]
        f1value, acc, score = train_lightGBM(X_train, Y_train,X_test, Y_test)
        print("Fold "+str(k) +":")
        print("f1:" + str(f1value) +" Accuracy: "+ str(acc) + " Score: "+str(score))
        f1Num.append(f1value)
        accNum.append(acc)
        scoreNum.append(score)
    
    print("The f1 of the vectors is : \n")
    print(str(np.sum(f1Num) / n_splits) + " f1  std : " + str(np.std(f1Num)))
    print("------------")
    print("The accuracy of the vectors is : \n")
    print(str(np.sum(accNum) / n_splits) + " accuracy std: " + str(np.std(accNum)))
    print("------------")
    print("The score of the vectors is : \n")
    print(str(np.sum(scoreNum) / n_splits) + " score std: " + str(np.std(scoreNum)))
    print("*****************")
    
def train_lightGBM(X_train, y_train,X_test, y_test):
    model_lgb = lgb.LGBMClassifier(boosting_type='goss', num_leaves=1000, learning_rate=0.01, n_estimators=2000, max_depth=-1,
                                   metric='multiclass',min_child_samples=30)
    
    model_lgb.fit(X_train,y_train)
    y_pred = model_lgb.predict(X_test)
    
    f1value = f1_score(y_test, y_pred,average="weighted")
    acc = accuracy_score(y_test, y_pred)
    score = model_lgb.score(X_test, y_test)
    return f1value, acc, score

def train_lightGBM_cv(train_data):
    params = {
        'objective': 'binary',
        'learning_rate': 0.1,
        'num_leaves': 31,
        'tree_learner':'feature',
        'num_threads': 6,
        'metric': 'binary_logloss',
        'max_depth': -1,
        'min_child_samples': 25
    }  # 设置出参数
    
    num_round = 10
    
    model = lgb.cv(params = params, train_set = train_data, num_boost_round = 200, 
           nfold=5,early_stopping_rounds=10)
    print(model.best_params_, model.best_score_)


def classify(index):
    all_senvector1 = [];
    Labels = []
    labelid = []
    
    for line in index:
        linelist = line.lstrip().rstrip().split("##")
        label = linelist[0]
        if label in select_category:
            Labels.append(linelist[0])
            labelid.append(select_category.index(label))
            all_senvector1.append(np.array(linelist[1].lstrip().rstrip().split(),dtype='float64'))
    
    x_data = np.array(all_senvector1)
    y_data = np.array(labelid)
    
    print("begin to classify")
    
    #train_data = lgb.Dataset(data=x_data,label=y_data)
    
    X_train = pd.DataFrame(x_data,dtype = 'float')
    Y_train = pd.DataFrame(y_data,dtype = 'int')
    
    kfold(X_train,Y_train)
    
    #train_lightGBM(X_train,Y_train)
    print("done")
    

if __name__ == '__main__':
    
    if (len(sys.argv) != 5):
        print(" python *.py <biswtmcodesinputfile> <categoryfile> <biswtmfeaturesinputfile, doc_dis> <model_root>")
        print(" ")
        exit(0)
    
    biswtmcodesinputfile = open(sys.argv[1],"r", encoding = "utf-8")
    categoryfile = open(sys.argv[2],"r", encoding = "utf-8")
    biswtmfeaturesinputfile = open(sys.argv[3],"r", encoding = "utf-8")
    
    oDir = sys.argv[4]
    select_out_category = {}
    for line in biswtmcodesinputfile:
        biswtmcodes.append(line.rstrip().lstrip())
    for line in categoryfile:
        ca = line.rstrip().lstrip()
        categories.append(ca)
        if ca not in select_out_category:
            select_out_category.setdefault(ca, 1)
        else:
            select_out_category[ca] +=1
    log_sorted = sorted(select_out_category.items(), key=operator.itemgetter(1),  reverse=True)[:30]
    select_category = []
    for item in log_sorted:
        select_category.append(item[0])
    print(select_category)
    
    for line in biswtmfeaturesinputfile:
        biswtmfeatures.append(line.rstrip().lstrip())
        
    if len(categories) != len(biswtmcodes) or len(biswtmfeatures) != len(biswtmcodes):
        print("There exsits a bug, and exit now.")
        exit(0)
    
    rand_linedic = {}
    for i in range(len(biswtmcodes)):
        wordlist = biswtmcodes[i].split("@")[1]
        temp = wordlist.rstrip().lstrip().strip()
        temp2 = temp.split(" ")[1:]
        wc = 0 
        for w in temp2:
            wc += int(w.split(":")[1])
        
        if categories[i] in select_category:
            strvector = categories[i] + "##" + biswtmfeatures[i]
            rand_linedic.setdefault(strvector, wc)
    
    #ranked_linedic = sorted(rand_linedic.items(), key=operator.itemgetter(1),  reverse=False)
    #print(ranked_linedic)
    

    c1 = []
    d1 = []
    a1 = []
    b1 = []
    
    
    for k,v in rand_linedic.items():    
        if v > 20 and v <= 30:
            a1.append(k)
        elif v > 30 and v <= 40:
            b1.append(k)
        elif v > 40 and v < 50:
            c1.append(k)
        elif v >= 50 and v <= 10000:
            d1.append(k)
 
    
    print(len(c1))
    print(len(d1))
    print(len(a1))
    print(len(b1))

    random.shuffle(c1)
    random.shuffle(d1)
    random.shuffle(a1)
    random.shuffle(b1)
    
    a = a1[:1000]
    b = b1[:1000]
    c = c1[:1000]
    d = d1[:1000]

    classify(a)
    
    classify(b)
    
    classify(c)
    
    classify(d)
    
    print("end.")