'''
Created on May 30, 2021

@author: shuangyinli
'''

import lightgbm as lgb
from numpy import *
import numpy as np
import operator
import random
import copy
import sys
import re
import time
import os
from multiprocessing import Process, Manager
from copy import deepcopy
from collections import Counter
from sklearn.metrics import top_k_accuracy_score

from sklearn.metrics import precision_score, recall_score, f1_score,accuracy_score
from sklearn.model_selection import cross_validate
from sklearn.model_selection import GridSearchCV
from sklearn.model_selection import train_test_split, StratifiedKFold
from sklearn import metrics as ms
import pandas as pd
from sklearn.model_selection import KFold
from sklearn.model_selection import train_test_split
from sklearn import datasets


def kfold(X,Y, n_splits=5):
    kf = KFold(n_splits,shuffle=True,random_state=None)

    f1Num = []
    accNum = []
    scoreNum =[]
    acc_top_3Num =[]
    acc_top_5Num =[]
    for k, (train, test) in enumerate(kf.split(X,Y)):
        X_train = X.iloc[train]
        X_test = X.iloc[test]
        Y_train = Y.iloc[train]
        Y_test = Y.iloc[test]
        f1value, acc, score,acc_top_5,acc_top_3 = train_lightGBM(X_train, Y_train,X_test, Y_test)
        print("Fold "+str(k) +":")
        print("f1:" + str(f1value) +" Accuracy: "+ str(acc) + " Score: "+str(score)+" precision@ 5: "+str(acc_top_5)+" precision@ 3: "+str(acc_top_3))
        f1Num.append(f1value)
        accNum.append(acc)
        scoreNum.append(score)
        acc_top_5Num.append(acc_top_5)
        acc_top_3Num.append(acc_top_3)
    
    print("The f1 of the vectors is : \n")
    print(str(np.sum(f1Num) / n_splits) + " f1  std : " + str(np.std(f1Num)))
    print("------------")
    print("The accuracy of the vectors is : \n")
    print(str(np.sum(accNum) / n_splits) + " accuracy std: " + str(np.std(accNum)))
    print("------------")
    print("The score of the vectors is : \n")
    print(str(np.sum(scoreNum) / n_splits) + " score std: " + str(np.std(scoreNum)))
    print("------------")
    print("The top_5 acc of the vectors is : \n")
    print(str(np.sum(acc_top_5Num) / n_splits) + " top_5 acc std: " + str(np.std(acc_top_5Num)))
    print("------------")
    print("The top_3 acc of the vectors is : \n")
    print(str(np.sum(acc_top_3Num) / n_splits) + " top_3 acc std: " + str(np.std(acc_top_3Num)))
    print("***********************************")
    
def train_lightGBM(X_train, y_train,X_test, y_test):
    model_lgb = lgb.LGBMClassifier(boosting_type='goss', num_leaves=1000, learning_rate=0.01, n_estimators=2000, max_depth=-1,
                                   metric='multiclass',min_child_samples=30)
    
    model_lgb.fit(X_train,y_train)
    y_pred = model_lgb.predict(X_test)
    #y_score = model_lgb.score(X_train, y_train, sample_weight)
    y_proba = model_lgb.predict_proba(X_test)
    
    f1value = f1_score(y_test, y_pred,average="weighted")
    score = model_lgb.score(X_test, y_test)
    acc = accuracy_score(y_test, y_pred)
    acc_top_5 = top_k_accuracy_score(y_test,y_proba, k=5)
    acc_top_3 = top_k_accuracy_score(y_test,y_proba, k=3)
    
    return f1value, acc, score, acc_top_5, acc_top_3

def train_lightGBM_cv(train_data):
    params = {
        'objective': 'binary',
        'learning_rate': 0.1,
        'num_leaves': 31,
        'tree_learner':'feature',
        'num_threads': 6,
        'metric': 'binary_logloss',
        'max_depth': -1,
        'min_child_samples': 25
    }  # 设置出参数
    
    num_round = 10
    
    model = lgb.cv(params = params, train_set = train_data, num_boost_round = 200, 
           nfold=5,early_stopping_rounds=10)
    print(model.best_params_, model.best_score_)


if __name__ == '__main__':
    
    if (len(sys.argv) != 2):
        print(" python build <dropoutfile> <model_root>")
        print(" ")
        exit(0)
    
    openfile = open(sys.argv[1], "r", encoding = "utf-8")
    ##
    select_out_category = {}
    for line in openfile:
        ca = line.strip().split("##")[0]
        if ca not in select_out_category:
            select_out_category.setdefault(ca, 1)
        else:
            select_out_category[ca] +=1
    log_sorted = sorted(select_out_category.items(), key=operator.itemgetter(1),  reverse=True)[:30]
    
    select_category = []
    for item in log_sorted:
        select_category.append(item[0])
    print(select_category)
    
    all_senvector1 = []
    Labels = []
    labelid = []
    openfile.seek(0)
    
    for line in openfile:
        linelist = line.lstrip().rstrip().split("##")
        label = linelist[0]
        if label in select_category:
            Labels.append(linelist[0])
            labelid.append(select_category.index(label))
            all_senvector1.append(np.array(linelist[1].lstrip().rstrip().split(),dtype='float64'))
    
    x_data = np.array(all_senvector1)
    y_data = np.array(labelid)
    
    print("begin to classify")
    
    #train_data = lgb.Dataset(data=x_data,label=y_data)
    
    X_train = pd.DataFrame(x_data,dtype = 'float')
    Y_train = pd.DataFrame(y_data,dtype = 'int')
    
    kfold(X_train,Y_train)
    
    #train_lightGBM(X_train,Y_train)
    print("done")
    
    pass